select 'sqrt(4 m^2)'::unit;
select 'sqrt(-4 m^2)'::unit;
select 'sqrt(4 m^3)'::unit;

select 'exp(1)'::unit;
select 'exp(1 A)'::unit;

select 'ln(2.7)'::unit;
select 'ln(-2.7)'::unit;
select 'ln(2.7 s)'::unit;

select 'asin(1)'::unit;
select 'asin(2)'::unit;
select 'asin(1 kg)'::unit;

select 'tan(1)'::unit;
select 'tan(1 mol)'::unit;
