SET unit.output_superscript = on;

SELECT 'm⁰'::unit;
SELECT 'm¹'::unit;
SELECT 'm²'::unit;
SELECT 'm⁻²'::unit;
SELECT 'm³'::unit;
SELECT 'm⁴'::unit;
SELECT 'm⁵'::unit;
SELECT 'm⁶'::unit;
SELECT 'm⁷'::unit;
SELECT 'm⁸'::unit;
SELECT 'm⁹'::unit;
SELECT 'm³⁴'::unit;
SELECT 'm⁻⁵⁶'::unit;
SELECT 'm⁺⁷⁸'::unit;
